// Souboe MATICE.CS
// Ukazuje petovn opertor
//
// Tda Matice reprezentuje matici typu 2 x 2 (2 dky, dva sloupce)
// Pro ni definujeme rzn opertory
// Tda Matice obsahuje konstantu d, udvajc poet dk a sloupc,
// dle pole typu double[2,2], obsahujc prvky matice, a statick datov sloky
// Nula a I, pedstavujc nulovou a jednotkovou matici
// Sloky Nula a I jsou readonly a inicializuje je statick konstruktor
// Vedle konstruktor obsahuje tato tda ladic metodu Vypi(), je vype 
// matici do standardnho vstupu

#define LADM

using System;
using System.Diagnostics;

public class Matice: ICloneable
{
	const int d = 2;
	double[,] pole = new  double [d, d];
	public static readonly Matice Nula;
	public static readonly Matice I;
	// Statick konstruktory
	static Matice()
	{
		Nula = new Matice();
		I = new Matice(1);
	}
	// Instann konstruktory
	public Matice(){}									// Vytvo nulovou matici
	public Matice(double d)								// Vytvo matici s d na diagonle
	{
		for(int i = 0; i < d; i++) pole[i,i] = d;
	}
	public Matice(double a00, double a01, double a10, double a11)// Vytvo matici se zadanmi 
	{															// hodnotami prvk
		pole[0,0] = a00; pole[0,1] = a01;
		pole[1,0] = a10; pole[1,1] = a11;
	}

	[Conditional("LADM")]								// Podmnn metoda pro vpis matice
	public void Vypi()									// do standardnho vstupu
	{
		for(int i = 0; i < d; i++)
		{
			Console.Write("|");
			for(int j = 0; j < d; j++)
				Console.Write("{0:F3} ", pole[i,j]);
			Console.WriteLine("|");
		}
	}

	// Klonovn matic
	public virtual object Clone()
	{
		Matice C = (Matice)this.MemberwiseClone();
		C.pole = new double[d, d];
		for(int i = 0; i < d; i++)
			for(int j = 0; j < d; j++)
				C.pole[i, j] = this.pole[i,j];
		return C;
	}
	
	// Metoda Equals() porovn aktuln instanci s jakmkoli jinm objektem
	// Nejprve testuje, zda je pedan parametr Matice; pokud ne, vrt false
	// Pokud to je matice, porovn prvky pol, a pokud naraz na rozdl, vrt false
	// jinak vrt true
	public override bool Equals(object Obj)
	{
		if(Obj is Matice)
		{
			Matice M = (Matice)Obj;
			for(int i = 0; i < d; i++)
				for(int j = 0; j < d; j++)
					if(M.pole[i, j] != this.pole[i,j]) return false;
			return true;
		}
		else return false;
	}
	// Opertor + pro stn matic
	// Stn je definovno po prvcch (staj se prvky se stejnmi indexy)
	public static Matice operator+(Matice A, Matice B)
	{
		Matice C = new Matice();
		for(int i = 0; i < d; i++)
			for(int j = 0; j < d; j++)
				C.pole[i,j] = A.pole[i,j] + B.pole[i,j];
		return C;
	}
	
	// Unrn opertor - pro vytvoen matice s prvky s opanm znamnkem
	public static Matice operator-(Matice A)
	{
		Matice C = new Matice();
		for(int i = 0; i < d; i++)
			for(int j = 0; j < d; j++)
				C.pole[i,j] = -A.pole[i,j];
		return C;
	}
	
	
	// Opertor - pro odtn matic
	// Odtn je definovno po prvcch (odetaj se prvky se stejnmi indexy)
	public static Matice operator-(Matice A, Matice B)
	{
		Matice C = new Matice();
		for(int i = 0; i < d; i++)
			for(int j = 0; j < d; j++)
				C.pole[i,j] = A.pole[i,j] - B.pole[i,j];
		return C;
	}

	// Opertor * pro nsoben matic
	// nsoben matic je definovno takto: (i,j)t prvek souinu se rovn 
	// skalrnmu souinu i-tho dku a j-tho sloupce
	public static Matice operator*(Matice A, Matice B)
	{
		Matice C = new Matice();
		for(int i = 0; i < d; i++)
			for(int j = 0; j < d; j++)
				for(int k = 0; k < d; k++)
					C.pole[i,j] += A.pole[i,k] * B.pole[k,j];
		return C;
	}

	// Opertory pro nsoben matice slem
	public static Matice operator*(double d, Matice A)
	{
		Matice C = new Matice();
		for(int i = 0; i < d; i++)
			for(int j = 0; j < d; j++)
				C.pole[i,j] = A.pole[i,j]*d;
		return C;
	}

	public static Matice operator*(Matice A, double d)
	{
		return d * A;
	}

	// Indexer zpstupujc prvku matice
	public double this[int i, int j]
	{	
		set{pole[i,j] = value;}
		get{return pole[i,j];}
	}

	// Determinant matice 
	// (Jaksi slo piazen kad tvercov matici, vypovd nco o vlasnostech matice - 
	// nap. jestli je tato matice regulrn atd.)
	public double Det
	{
		get{return pole[0, 0]*pole[1, 1] - pole[0, 1]*pole[1, 0];}
	}

	// Opertory == a != vyuvaj metodu Equals()
	public static bool operator == (Matice A, Matice B)
	{
		return A.Equals(B);
	}
	public static bool operator != (Matice A, Matice B)
	{
		return !A.Equals(B);
	}

	// Petypovn sla na diagonln matici
	public static explicit operator Matice (double d){return new Matice(d);}	
}

class Program
{
	public static void Main()
	{
		Matice A = new Matice(2), B = new Matice(1,1,1,1);
		Matice C = A + B;
		C.Vypi();
		Matice D = (Matice)B.Clone();
		bool b = B.Equals(D);
		B[1,1] = 7;
		b = B.Equals(D);
		D.Vypi();
		C = 5*A;
		C = A * 5;
		C.Vypi();
	}
}
